package org.imixs.workflow.jee.rest.jaxrs;

import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Collection;
import java.util.Vector;

import javax.ejb.EJB;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.EntityTable;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * The WorkflowService Handler supports methods to process different kind of
 * request URIs
 * 
 * @author rsoika
 * 
 */
@Path("/report")
@Produces( { "text/html", "application/xml", "application/json" })
public class ReportService {

	@EJB
	org.imixs.workflow.jee.ejb.EntityService entityService;

	@EJB
	org.imixs.workflow.jee.ejb.ReportService reportService;

	@GET
	@Path("/reports")
	public EntityCollection getAllReports(
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count) {
		try {
			Collection<ItemCollection> col = null;
			try {
				col = reportService.getReportList(start, count);
				return XMLItemCollectionAdapter.putCollection(col);
			} catch (Exception e) {
				e.printStackTrace();
			}
			return new EntityCollection();

		} catch (Exception e) {
			e.printStackTrace();
		}
		return new EntityCollection();
	}

	/**
	 * Returns a single report by name or by uniqueid
	 * 
	 * @param name
	 *            reportname or uniqueid of report
	 * @return
	 */
	@GET
	@Path("/reports/{name}")
	public XMLItemCollection getReport(@PathParam("name") String name) {
		try {
			ItemCollection itemCol = reportService.getReport(name);
			return XMLItemCollectionAdapter.putItemCollection(itemCol);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Executes a single report defined by name or uniqueid
	 * 
	 * The output depends on the requested media format
	 * 
	 * 
	 * @param name
	 *            reportname of the report to be executed
	 * @return a collection of entiteis
	 * 
	 */
	@GET
	@Path("/{name}.ixr")
	public Response getExcecuteReport(@PathParam("name") String name,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count) {
		Collection<ItemCollection> col = null;
		String reportName = null;
		String sEQL;
		String sXSL;
		String sContentType;
		String sEncoding;
		try {

			reportName = name + ".ixr";

			ItemCollection itemCol = reportService.getReport(reportName);

			// get Query and output format
			sEQL = itemCol.getItemValueString("txtquery");
			sXSL = itemCol.getItemValueString("txtXSL").trim();
			sContentType = itemCol.getItemValueString("txtcontenttype");
			if ("".equals(sContentType))
				sContentType="text/html";
			
			
			sEncoding = itemCol.getItemValueString("txtencoding");
			// no encoding defined so take a default encoding
			// (UTF-8)
			if ("".equals(sEncoding))
				sEncoding = "UTF-8";
			
			
			// query result
			col = entityService.findAllEntities(sEQL, start, count);

			// if no XSL is provided return standard html format...?
			if ("".equals(sXSL)) {
				Response.ResponseBuilder builder = Response.ok(XMLItemCollectionAdapter.putCollection(col),
					"text/html");
				return builder.build();	
			}
			
			// Transform XML per XSL and generate output
			EntityCollection xmlCol=XMLItemCollectionAdapter.putCollection(col);


			StringWriter writer = new StringWriter();

			JAXBContext context = JAXBContext.newInstance(EntityCollection.class);
			Marshaller m=context.createMarshaller();
			m.marshal(xmlCol,writer);
			
			StringWriter resultWriter=new StringWriter();
			
			xslTranformation(writer.toString(),sXSL,sEncoding,resultWriter);
			
			resultWriter.flush();
			// TODO   maybe the output writer should be a byte stream object ?
			Response.ResponseBuilder builder = Response.ok(resultWriter.toString(),
					sContentType);

			return builder.build();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@GET
	@Produces("text/html")
	@Path("/{name}.html")
	public EntityTable getExcecuteReportHTML(@PathParam("name") String name,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count) {
		Collection<ItemCollection> col = null;
		String reportName = null;

		try {

			reportName = name + ".ixr";

			ItemCollection itemCol = reportService.getReport(reportName);

			String sEQL = itemCol.getItemValueString("txtquery");
			Vector<String> vAttributList = itemCol
					.getItemValue("txtAttributeList");
			col = entityService.findAllEntities(sEQL, start, count);

			EntityCollection entityCol = XMLItemCollectionAdapter
					.putCollection(col);
			EntityTable entityTable = new EntityTable();
			entityTable.setAttributeList(vAttributList);
			entityTable.setEntity(entityCol.getEntity());

			return entityTable;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Returns a xml stream from a report execution
	 * 
	 * @param name
	 *            reportname of the report to be executed
	 * @param start
	 * @param count
	 * @return
	 * @throws Exception
	 */
	@GET
	@Produces("application/xml")
	@Path("/{name}.xml")
	public EntityCollection getExcecuteReportXML(
			@PathParam("name") String name,
			@DefaultValue("0") @QueryParam("start") int start,
			@DefaultValue("10") @QueryParam("count") int count)
			throws Exception {
		String reportName = null;
		Collection<ItemCollection> col = null;

		try {

			reportName = name + ".ixr";

			ItemCollection itemCol = reportService.getReport(reportName);

			String sEQL = itemCol.getItemValueString("txtquery");
			col = entityService.findAllEntities(sEQL, start, count);

			return XMLItemCollectionAdapter.putCollection(col);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Deletes a report by name or by uniqueid
	 * 
	 * @param name
	 *            of report or uniqueid
	 */
	@DELETE
	@Path("/reports/{name}")
	public void deleteModel(@PathParam("version") String name) {
		try {
			ItemCollection itemCol = reportService.getReport(name);
			entityService.remove(itemCol);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * This method updates or creates a Report object provided in a
	 * XMLItemCollection object
	 * 
	 * 
	 * @param reportCol
	 *            - report data
	 */
	@PUT
	@Consumes( { "application/xml", "text/xml" })
	public void putReport(XMLItemCollection reportCol) {
		ItemCollection itemCollection;
		try {
			itemCollection = XMLItemCollectionAdapter
					.getItemCollection(reportCol);
			reportService.updateReport(itemCollection);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@POST
	@Consumes( { "application/xml", "text/xml" })
	public void postReport(XMLItemCollection reportCol) {
		putReport(reportCol);
	}

	
	
	
	
	/**
	 * This method transforms an xml string with a provided xsl String
	 * 
	 * The XMLSource will be read from a InputStreamReader object to provide the
	 * XSLTransformer with a inputsream in expected encoding.
	 * 
	 * TODO mabe that the outputwriter shoulb be better a byte stream ??  
	 * 
	 * 
	 * @param xmlSource
	 * @param xsltSourceFile
	 * @return
	 */
	
	protected void xslTranformation(String xmlSource, String xslSource,
			String aEncoding,StringWriter outputWriter) {
		try {
			TransformerFactory transFact = TransformerFactory.newInstance();

			// generate XML InputStream Reader with encoding
			ByteArrayInputStream baisXML = new ByteArrayInputStream(xmlSource
					.getBytes());
			InputStreamReader isreaderXML;
			
				isreaderXML = new InputStreamReader(baisXML,
						aEncoding);
			
			Source xmlSrc = new StreamSource(isreaderXML);

			// generate XSL InputStream Reader with encoding
			ByteArrayInputStream baisXSL = new ByteArrayInputStream(xslSource
					.getBytes());
			InputStreamReader isreaderXSL = new InputStreamReader(baisXSL,
					aEncoding);
			Source xslSrc = new StreamSource(isreaderXSL);

			Transformer trans = transFact.newTransformer(xslSrc);
			//OutputStream outStream = uriHandler.getResponse().getOutputStream();
			trans.transform(xmlSrc, new StreamResult(outputWriter));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
}
